/** simext.c Copyright 1992-2010 Ventana Systems, Inc.
Version 5.10b
**
**  ABSTRACT:
**      This file contains the supporting code for directly called 
        functions used - or potentially used - in mdl.c
**  AUTHORS:
**
**      Robert L. Eberlein
**/
/*******************************************************************
  COMPILER NOTES

  This is a standard C program and should work with any compiler though 
  it is only tested with Microsoft C.  

  To use this with the double precision version of Vensim it is 
  necessary to define DBLPREC and recompile.  

  
********************************************************************/

#include "vensim.h"
GLOB_VARS *VGV ;

/*************************************************************************

  Part 2

  Function prototypes and macro definitions for use in compiled simulations.
  Note that you can call the exported functions listed here from external
  functions as well if that is at all helpful.

  ********************************************************************/


#define TINY 1e-6 
/* functions defined as macros to work more easily */ 
#define FIRST_IF_TRUE(a,b,c) ((a)?(b):(c)) 
#define IF_THEN_ELSE(a,b,c) ((a)?(b):(c)) 
//#define ABS(a) ((a)>0?(a):-(a)) 
#define ABS(a) fabs(a) /* single evaluation of the argument */
#ifndef SIMEXT
//#define MAX(a,b) ((a) > (b)?(a):(b)) 
__inline BigREAL MAX(BigREAL a,BigREAL b)
 {
  if(a > b)
     return a ;
  return b ;
  }
//#define MIN(a,b) ((a) > (b)?(b):(a)) 
__inline BigREAL MIN(BigREAL a,BigREAL b)
 {
  if(a < b)
     return a ;
  return b ;
  }
#endif


  
#define LOG(a,b) (log(a)/log(b)) 
#define LN(a)  (log(a)) 
/* XIDZ and ZIDZ need to behave the same as for interpreted which means we need
   an extra argument and a max nesting depth (set at 10) */
#define XIDZ(n,a,b,c) ((temp[n]=(b)) <= TINY && temp[n] > -TINY?(c):(a)/temp[n])
#define ZIDZ(n,a,b) ((temp[n]=(b)) <= TINY && temp[n] > -TINY?0:(a)/temp[n])
#define EXP(a) exp(a) 
#define COS(a) cos(a) 
#define COSH(a) cosh(a) 
#define SIN(a) sin(a) 
#define SINH(a) sinh(a) 
#define ARCCOS(a) acos(a) 
#define ARCSIN(a) asin(a) 
#define ARCTAN(a) atan(a) 
#define TAN(a) tan(a) 
#define TANH(a) tanh(a) 
#define SQRT(a) sqrt(a) 
#define INTEGER(a) ((COMPREAL)((long)(a)))

/* functions used in models */ 
CFUNCTION BigREAL TABLE(TAB_TYPE *tab,BigREAL in) ; 
CFUNCTION BigREAL LOOKUP_BACKWARD(TAB_TYPE *tab,BigREAL in) ; 
CFUNCTION BigREAL LOOKUP_FORWARD(TAB_TYPE *tab,BigREAL in) ; 
CFUNCTION BigREAL LOOKUP_INVERT(TAB_TYPE *tab,BigREAL in) ; 
CFUNCTION BigREAL LOOKUP_EXTRAPOLATE(TAB_TYPE *tab,BigREAL in) ; 
CFUNCTION BigREAL LOOKUP_AREA(TAB_TYPE *tab,BigREAL start,BigREAL end) ; 
CFUNCTION BigREAL LOOKUP_SLOPE(TAB_TYPE *tab,BigREAL x,BigREAL mode) ; 
CFUNCTION BigREAL MESSAGE(char *msg,BigREAL severity) ;
CFUNCTION BigREAL RANDOM_0_1 (void) ; 
CFUNCTION BigREAL RANDOM_BETA(BigREAL xmin,BigREAL xmax,BigREAL alpha,BigREAL beta,BigREAL shift,BigREAL stretch,BigREAL streamid) ;
CFUNCTION BigREAL RANDOM_BINOMIAL(BigREAL xmin,BigREAL xmax,BigREAL p,BigREAL n,BigREAL shift,BigREAL stretch,BigREAL streamid) ;
CFUNCTION BigREAL RANDOM_EXPONENTIAL(BigREAL xmin,BigREAL xmax,BigREAL shift,BigREAL stretch,BigREAL streamid) ;
CFUNCTION BigREAL RANDOM_GAMMA(BigREAL xmin,BigREAL xmax,BigREAL a,BigREAL shift,BigREAL stretch,BigREAL streamid) ;
CFUNCTION BigREAL RANDOM_LOOKUP(TAB_TYPE *look,BigREAL xmin,BigREAL xmax,BigREAL shift,BigREAL stretch,BigREAL streamid) ;
CFUNCTION BigREAL RANDOM_NEGATIVE_BINOMIAL(BigREAL xmin,BigREAL xmax,BigREAL p,BigREAL n,BigREAL shift,BigREAL stretch,BigREAL streamid) ;
CFUNCTION BigREAL RANDOM_NORMAL(BigREAL xmin,BigREAL xmax,BigREAL shift,BigREAL stretch,BigREAL streamid) ;
CFUNCTION BigREAL RANDOM_POISSON(BigREAL xmin,BigREAL xmax,BigREAL a,BigREAL shift,BigREAL stretch,BigREAL streamid) ;
CFUNCTION BigREAL RANDOM_TRIANGULAR(BigREAL xmin,BigREAL xmax,BigREAL peak,BigREAL shift,BigREAL stretch,BigREAL streamid) ;
CFUNCTION BigREAL RANDOM_UNIFORM(BigREAL xmin,BigREAL xmax,BigREAL streamid) ;
CFUNCTION BigREAL RANDOM_WEIBULL(BigREAL xmin,BigREAL xmax,BigREAL shape,BigREAL shift,BigREAL stretch,BigREAL streamid) ;
CFUNCTION BigREAL QUANTUM(BigREAL a,BigREAL b) ; 
CFUNCTION BigREAL MODULO(BigREAL a,BigREAL b) ; 
CFUNCTION BigREAL STEP(BigREAL a,BigREAL b) ; 
CFUNCTION BigREAL GAMMA_LN(BigREAL a) ; 
CFUNCTION BigREAL RAMP(BigREAL a,BigREAL b,BigREAL c) ; 
CFUNCTION BigREAL PULSE(BigREAL ftime,BigREAL width) ; 
CFUNCTION BigREAL PULSE_TRAIN(BigREAL  ftime,BigREAL  width,BigREAL repeattime,BigREAL stop) ;
CFUNCTION BigREAL SHIFT_IF_TRUE(VECTOR_ARG *array,BigREAL cond,BigREAL arrsize,BigREAL cumulate,
                 BigREAL startval) ;
CFUNCTION BigREAL MARKETP(VECTOR_ARG *request,VECTOR_ARG *priority,int size,BigREAL width,BigREAL supply) ; 
CFUNCTION BigREAL ALLOCATE_BY_PRIORITY(VECTOR_ARG *allocation,VECTOR_ARG *request,VECTOR_ARG *priority,int size,BigREAL width,BigREAL supply) ; 

CFUNCTION BigREAL ALLOCATE_AVAILABLE(VECTOR_ARG *allocation,VECTOR_ARG *request,VECTOR_ARG *request_p,BigREAL available) ;
CFUNCTION BigREAL MARKET_MATCH(VECTOR_ARG *sales,VECTOR_ARG *demandq,VECTOR_ARG *dshape,VECTOR_ARG *seller_attract,VECTOR_ARG *supplyq,VECTOR_ARG *sshape,VECTOR_ARG *buyer_priority,VECTOR_ARG *dqsconversion) ;
CFUNCTION BigREAL FIND_MARKET_PRICE(VECTOR_ARG *d,VECTOR_ARG *d_p,VECTOR_ARG *s,VECTOR_ARG *sp) ;
/* DEMAND AT PRICE and SUPPLY AT PRICE are really the same function */
#define DEMAND_AT_PRICE(ds,ds_p,p) MARKET_DS(ds,ds_p,p,1)
#define SUPPLY_AT_PRICE(ds,ds_p,p) MARKET_DS(ds,ds_p,p,0)
CFUNCTION BigREAL MARKET_DS(VECTOR_ARG *ds,VECTOR_ARG *ds_p,BigREAL price,int isdemand) ;

CFUNCTION BigREAL VECTOR_REORDER(VECTOR_ARG *outvector,VECTOR_ARG *invector,VECTOR_ARG *sortvector) ;
CFUNCTION BigREAL VECTOR_SORT_ORDER(VECTOR_ARG *outvector,VECTOR_ARG *invector,BigREAL d) ;
CFUNCTION BigREAL VECTOR_RANK(VECTOR_ARG *outvector,VECTOR_ARG *invector,BigREAL d) ;
CFUNCTION BigREAL ALLOC_P(BigREAL request,BigREAL priority,BigREAL width,BigREAL mp) ; 
CFUNCTION BigREAL TIME_BASE(BigREAL intecept,BigREAL slope) ; 
CFUNCTION BigREAL INIT_PURE_DELAY(int type,VAL_OFF deloff,VAL_OFF valoff,double deltime,
                   double initval,double missval_or_order,TAB_TYPE *tab,TAB_TYPE *tab2) ;
CFUNCTION BigREAL STORE_PURE_DELAY(VAL_OFF deloff,double val,double outval,
        double delaytime,double changerate,TAB_TYPE *tab) ;
CFUNCTION BigREAL INIT_SINTEG(int type,
                   VAL_OFF deloff,VAL_OFF valoff,double initval,
                   double minval,double maxval,double quantval,
                   double specval,double tolerance) ;
CFUNCTION BigREAL SAMPLE_IF_TRUE(double oldval,double cond,double newval) ;
CFUNCTION BigREAL FIND_ZERO(VECTOR_ARG *vx,VECTOR_ARG *vsx,VECTOR_ARG *vz,VECTOR_ARG *vsz,VECTOR_ARG *inix,BigREAL fn,
      BigREAL fns,BigREAL tolerance,BigREAL fmax_iter,BigREAL method) ;
CFUNCTION BigREAL INVERT_MATRIX(VECTOR_ARG *ainv,VECTOR_ARG *a,BigREAL n) ;
CFUNCTION BigREAL POWER(BigREAL base,BigREAL exponent) ;
CFUNCTION BigREAL VECTOR_ELM_MAP(VECTOR_ARG *in,BigREAL shift) ;
CFUNCTION BigREAL VECTOR_LOOKUP(VECTOR_ARG *in,BigREAL x,BigREAL xmin,BigREAL xmax,BigREAL mode) ;                 
CFUNCTION BigREAL GET_DATA_AT_TIME(VECTOR_ARG *v,BigREAL t) ;
CFUNCTION BigREAL GET_DATA_BETWEEN_TIMES(VECTOR_ARG *v,BigREAL t,BigREAL mode) ;
CFUNCTION BigREAL GET_DATA_ATTRIBUTE(int attrib,VECTOR_ARG *v,BigREAL t,BigREAL mode) ;
CFUNCTION BigREAL GET_DATA_FIRST_TIME(VECTOR_ARG *v) ;
CFUNCTION BigREAL GET_DATA_LAST_TIME(VECTOR_ARG *v) ;
CFUNCTION BigREAL GET_DATA_TOTAL_POINTS(VECTOR_ARG *v) ;
CFUNCTION BigREAL GET_TIME_VALUE(BigREAL relativeto,BigREAL offset,BigREAL measure) ;
CFUNCTION BigREAL QUEUE_AGE_AVERAGE(VECTOR_ARG *v,BigREAL x) ;
CFUNCTION BigREAL QUEUE_AGE_OLDEST(VECTOR_ARG *v) ;
CFUNCTION BigREAL QUEUE_AGE_IN_RANGE(VECTOR_ARG *v,BigREAL l,BigREAL h) ;
CFUNCTION BigREAL QUEUE_ATTRIB_AVERAGE(VECTOR_ARG *v,BigREAL x) ;
CFUNCTION BigREAL QUEUE_ATTRIB_MAX(VECTOR_ARG *v) ;
CFUNCTION BigREAL QUEUE_ATTRIB_MIN(VECTOR_ARG *v) ;
CFUNCTION BigREAL QUEUE_ATTRIB_IN_RANGE(VECTOR_ARG *v,BigREAL l,BigREAL h) ;
CFUNCTION BigREAL QUEUE_ATTRIB_QUANTITY(VECTOR_ARG *v,BigREAL x) ;

/* delay functions - defined as macros then passed back through
   an initial and active function */
#define DELAY_FIXED_i(type,varoff,delind,deltime,initval) INIT_PURE_DELAY(type,delind,varoff,deltime,initval,0.0,'\0','\0') 
#define DELAY_FIXED_a(delind,inval) STORE_PURE_DELAY(delind,inval,0.0,1.0,0,'\0')
#define DELAY_INFORMATION_i(type,varoff,delind,initval) INIT_PURE_DELAY(type,delind,varoff,1.0,initval,0.0,'\0','\0') 
#define DELAY_INFORMATION_a(delind,inval,deltime) STORE_PURE_DELAY(delind,inval,0.0,deltime,0,'\0')
#define DELAY_MATERIAL_i(type,varoff,delind,initval,missval) INIT_PURE_DELAY(type,delind,varoff,1.0,initval,missval,'\0','\0') 
#define DELAY_MATERIAL_a(delind,inval,deltime) STORE_PURE_DELAY(delind,inval,0.0,deltime,0,'\0')
/* these are not in 4.0f but will be going in future versions */
#define DELAY_N_i(type,varoff,delind,initval,order) INIT_PURE_DELAY(type,delind,varoff,1.0,initval,order,'\0','\0')
#define DELAY_N_a(delind,inval,deltime)  STORE_PURE_DELAY(delind,inval,0.0,deltime,0,'\0')
#define SMOOTH_N_i(type,varoff,delind,initval,order) INIT_PURE_DELAY(type,delind,varoff,1.0,initval,order,'\0','\0')
#define SMOOTH_N_a(delind,inval,deltime)  STORE_PURE_DELAY(delind,inval,0.0,deltime,0,'\0')
#define QUEUE_FIFO_i(type,varoff,delind,initprofile,initval,initage) INIT_PURE_DELAY(type,delind,varoff,initage,initval,0.0,initprofile,'\0')
#define QUEUE_FIFO_a(delind,inval,outval) STORE_PURE_DELAY(delind,inval,outval,1.0,0,'\0') ;
#define QUEUE_FIFO_ATTRIB_i(type,varoff,delind,initprofile,inattribprofile,initval,initattrib,initage) INIT_PURE_DELAY(type,delind,varoff,initage,initval,initattrib,initprofile,inattribprofile)
#define QUEUE_FIFO_ATTRIB_a(delind,inval,outval,inattrib,changerate) STORE_PURE_DELAY(delind,inval,outval,inattrib,changerate,'\0') 
#define DELAY_PROFILE_i(type,varoff,delind,initval,initgrowrate) INIT_PURE_DELAY(type,delind,varoff,1.0,initval,initgrowrate,'\0','\0') 
#define DELAY_PROFILE_a(delind,profile,inval,deltime) STORE_PURE_DELAY(delind,inval,0.0,deltime,0,profile)
#define DELAY_CONVEYOR_i(type,varoff,delind,initprofile,inittotal,initctime) INIT_PURE_DELAY(type,delind,varoff,initctime,inittotal,0,initprofile,'\0')
#define DELAY_CONVEYOR_a(delind,inval,deltime,drainage) STORE_PURE_DELAY(delind,inval,drainage,deltime,0,'\0')
#define DELAY_BATCH_i(type,varoff,delind,initbatch,inittime,initbacklog) INIT_PURE_DELAY(type,delind,varoff,inittime,initbatch,initbacklog,'\0','\0')
#define DELAY_BATCH_a(delind,inval,bsize,deltime) STORE_PURE_DELAY(delind,inval,bsize,deltime,0,'\0')
#define INTERNAL_RATE_OF_RETURN_i(type,varoff,delind,fperiod,initq,initt) INIT_PURE_DELAY(type,delind,varoff,fperiod,initq,initt,'\0','\0')
#define INTERNAL_RATE_OF_RETURN_a(delind,inval) STORE_PURE_DELAY(delind,inval,0,0,0,'\0')
#define NET_PRESENT_VALUE_i(type,varoff,delind,fperiod,fstart,foff) INIT_PURE_DELAY(type,delind,varoff,fperiod,fstart,foff,'\0','\0')

#define NET_PRESENT_VALUE_a(delind,inval,dr) STORE_PURE_DELAY(delind,inval,dr,0,0,'\0')
#define DEPRECIATE_BY_SCHEDULE_i(type,varoff,delind,sched,init,it,fstart) INIT_PURE_DELAY(type,delind,varoff,it,fstart,0,(TAB_TYPE *)sched,(TAB_TYPE *)init)
#define DEPRECIATE_BY_SCHEDULE_a(delind,inval) STORE_PURE_DELAY(delind,inval,0,0,0,'\0')
#define DEPRECIATE_STRAIGHTLINE_i(type,varoff,delind,deltime,fperiod,initq) INIT_PURE_DELAY(type,delind,varoff,deltime,initq,fperiod,'\0','\0')
#define DEPRECIATE_STRAIGHTLINE_a(delind,inval) STORE_PURE_DELAY(delind,inval,0,0,0,'\0')


CFUNCTION void* vensim_alloc_simmem(DATA_MATRIX *dm,CONSTANT_MATRIX *cm,V4BYTEU bytes) ;
CFUNCTION  BigREAL exp(BigREAL) ; 
CFUNCTION  BigREAL log(BigREAL) ; 
CFUNCTION  BigREAL sqrt(BigREAL) ; 
CFUNCTION  BigREAL sin(BigREAL) ; 
CFUNCTION  BigREAL sinh(BigREAL) ; 
CFUNCTION  BigREAL cos(BigREAL) ; 
CFUNCTION  BigREAL cosh(BigREAL) ; 
CFUNCTION  BigREAL tan(BigREAL) ; 
CFUNCTION  BigREAL tanh(BigREAL) ; 
CFUNCTION  BigREAL acos(BigREAL) ; 
CFUNCTION  BigREAL asin(BigREAL) ; 
CFUNCTION  BigREAL atan(BigREAL) ;

__inline VECTOR_ARG * __cdecl vecarg_assign(int ind,COMPREAL *vals)
{
   VGV->vector_arglist[ind].vals = vals ;
   return(VGV->vector_arglist + ind) ;
   }
/********************************************
 inline functions - simply calls to vensim exports 
 ***************************************************/

__inline BigREAL ALLOCATE_AVAILABLE(VECTOR_ARG *allocation,VECTOR_ARG *request,
      VECTOR_ARG *request_p,BigREAL available) {
  return (*VGV->ALLOCATE_AVAILABLE)(allocation,request,request_p,available) ;  }
__inline BigREAL MARKET_DS(VECTOR_ARG *ds,VECTOR_ARG *pprofile,BigREAL price,int isdemand){
   return (*VGV->MARKET_DS)(ds,pprofile,price,isdemand) ;   }
__inline BigREAL FIND_MARKET_PRICE(VECTOR_ARG *demand_q,VECTOR_ARG *demand_p,
                      VECTOR_ARG *supply_q,VECTOR_ARG *supply_p){
   return (*VGV->FIND_MARKET_PRICE)(demand_q,demand_p,supply_q,supply_p) ;   }
__inline BigREAL MARKET_MATCH (VECTOR_ARG *sales,
                      VECTOR_ARG *demandq,VECTOR_ARG *dshape,VECTOR_ARG *seller_attract,
                      VECTOR_ARG *supplyq,VECTOR_ARG *sshape,VECTOR_ARG *buyer_priority,
                      VECTOR_ARG *dsqconversion) {
   return (*VGV->MARKET_MATCH)(sales,demandq,dshape,seller_attract,
                      supplyq,sshape,buyer_priority,dsqconversion) ;  }
__inline BigREAL VECTOR_SORT_ORDER(VECTOR_ARG *outvector,VECTOR_ARG *invector,BigREAL d) {
 return (*VGV->VECTOR_SORT_ORDER)(outvector,invector,d) ; }
__inline BigREAL VECTOR_RANK(VECTOR_ARG *outvector,VECTOR_ARG *invector,BigREAL d) {
 return (*VGV->VECTOR_RANK)(outvector,invector,d) ; }
__inline BigREAL VECTOR_REORDER(VECTOR_ARG *outvector,VECTOR_ARG *invector,VECTOR_ARG *sortvector) {
 return (*VGV->VECTOR_REORDER)(outvector,invector,sortvector) ; }
__inline BigREAL GET_DATA_AT_TIME (VECTOR_ARG *dv,BigREAL t) {
 return (*VGV->GET_DATA_AT_TIME)(dv,t) ; }
__inline BigREAL GET_DATA_BETWEEN_TIMES (VECTOR_ARG *dv,BigREAL t,BigREAL mode) {
 return (*VGV->GET_DATA_BETWEEN_TIMES)(dv,t,mode) ;  }
__inline BigREAL GET_DATA_ATTRIBUTE (int attrib,VECTOR_ARG *dv,BigREAL t,BigREAL mode) {
 return (*VGV->GET_DATA_ATTRIBUTE)(attrib,dv,t,mode) ; }
__inline BigREAL GET_DATA_FIRST_TIME (VECTOR_ARG *dv) {
  return (*VGV->GET_DATA_FIRST_TIME)(dv) ; }
__inline BigREAL GET_DATA_LAST_TIME (VECTOR_ARG *dv) {
 return (*VGV->GET_DATA_LAST_TIME)(dv) ; }
__inline BigREAL GET_DATA_TOTAL_POINTS (VECTOR_ARG *dv) {
  return (*VGV->GET_DATA_TOTAL_POINTS)(dv) ; }
__inline BigREAL GET_TIME_VALUE(BigREAL relativeto,BigREAL offset,BigREAL measure) { 
  return (*VGV->GET_TIME_VALUE)(relativeto,offset,measure) ; }
__inline BigREAL MARKETP (VECTOR_ARG *request,VECTOR_ARG *priority,int size, BigREAL width,BigREAL supply) {
 return (*VGV->MARKETP)(request,priority,size,width,supply) ; }
__inline BigREAL MESSAGE (char *msg,BigREAL severity) { 
 return (*VGV->MESSAGE)(msg,severity) ; }
__inline BigREAL LOOKUP_AREA (TAB_TYPE *tab,BigREAL start,BigREAL end) {
 return (*VGV->LOOKUP_AREA)(tab,start,end) ; }
__inline BigREAL LOOKUP_SLOPE (TAB_TYPE *tab,BigREAL x,BigREAL mode) {
 return (*VGV->LOOKUP_SLOPE)(tab,x,mode) ; }
__inline BigREAL LOOKUP_FORWARD (TAB_TYPE *tab,BigREAL in) {
 return (*VGV->LOOKUP_FORWARD)(tab,in) ; }
__inline BigREAL LOOKUP_INVERT (TAB_TYPE *tab,BigREAL in) { 
 return (*VGV->LOOKUP_INVERT)(tab,in) ; }
__inline BigREAL LOOKUP_BACKWARD(TAB_TYPE *tab,BigREAL in) { 
 return (*VGV->LOOKUP_BACKWARD)(tab,in) ; }
__inline BigREAL LOOKUP_EXTRAPOLATE (TAB_TYPE *tab,BigREAL in) { 
 return (*VGV->LOOKUP_EXTRAPOLATE)(tab,in) ; }
__inline BigREAL TABLE (TAB_TYPE *tab,BigREAL in) {
 return (*VGV->TABLE)(tab,in) ; }
__inline BigREAL RANDOM_NEGATIVE_BINOMIAL(BigREAL xmin,BigREAL xmax,BigREAL p,BigREAL n,BigREAL shift,BigREAL stretch,BigREAL streamid) { 
 return (*VGV->RANDOM_NEGATIVE_BINOMIAL)(xmin,xmax,p,n,shift,stretch,streamid) ; }
__inline BigREAL RANDOM_NORMAL(BigREAL xmin,BigREAL xmax,BigREAL mean,BigREAL stddev,BigREAL streamid) { 
 return (*VGV->RANDOM_NORMAL)(xmin,xmax,mean,stddev,streamid) ; }
__inline BigREAL RANDOM_WEIBULL(BigREAL xmin,BigREAL xmax,BigREAL b,BigREAL shift,BigREAL stretch,BigREAL streamid){ 
 return (*VGV->RANDOM_WEIBULL)(xmin,xmax,b,shift,stretch,streamid) ; }
__inline BigREAL RANDOM_BETA(BigREAL xmin,BigREAL xmax,BigREAL a,BigREAL b,BigREAL shift,BigREAL stretch,BigREAL streamid) { 
 return (*VGV->RANDOM_BETA)(xmin,xmax,a,b,shift,stretch,streamid) ; }
__inline BigREAL RANDOM_BINOMIAL(BigREAL xmin,BigREAL xmax,BigREAL p,BigREAL n,BigREAL shift,BigREAL stretch,BigREAL streamid) { 
 return (*VGV->RANDOM_BINOMIAL)(xmin,xmax,p,n,shift,stretch,streamid) ; }
__inline BigREAL RANDOM_TRIANGULAR(BigREAL xmin,BigREAL xmax,BigREAL start,BigREAL peak,BigREAL stop,BigREAL streamid) { 
 return (*VGV->RANDOM_TRIANGULAR)(xmin,xmax,start,peak,stop,streamid) ; }
__inline BigREAL RANDOM_EXPONENTIAL(BigREAL xmin,BigREAL xmax,BigREAL shift,BigREAL stretch,BigREAL streamid) { 
 return (*VGV->RANDOM_EXPONENTIAL)(xmin,xmax,shift,stretch,streamid) ; }
__inline BigREAL RANDOM_LOOKUP(TAB_TYPE *look,BigREAL xmin,BigREAL xmax,BigREAL shift,BigREAL stretch,BigREAL streamid) { 
 return (*VGV->RANDOM_LOOKUP)(look,xmin, xmax,shift,stretch,streamid) ; }
__inline BigREAL RANDOM_POISSON(BigREAL xmin,BigREAL xmax,BigREAL x, BigREAL shift,BigREAL stretch,BigREAL streamid) { 
 return (*VGV->RANDOM_POISSON)(xmin,xmax,x,shift,stretch,streamid) ;  }
__inline BigREAL RANDOM_GAMMA(BigREAL xmin,BigREAL xmax,BigREAL x,BigREAL shift,BigREAL stretch,BigREAL streamid) { 
 return (*VGV->RANDOM_GAMMA)(xmin,xmax,x,shift,stretch,streamid) ; }
__inline BigREAL RANDOM_UNIFORM(BigREAL xmin,BigREAL xmax,BigREAL streamid) { 
 return (*VGV->RANDOM_UNIFORM)(xmin,xmax,streamid) ; }
__inline BigREAL RANDOM_0_1() { 
 return (*VGV->RANDOM_0_1)() ; }
__inline BigREAL QUEUE_AGE_AVERAGE(VECTOR_ARG *v,BigREAL x){ 
 return (*VGV->QUEUE_GETINFO)(v,x,0.0,F_QUEUE_AGE_AVERAGE) ; }
__inline BigREAL QUEUE_AGE_OLDEST(VECTOR_ARG *v){ 
 return (*VGV->QUEUE_GETINFO)(v,0.0,0.0,F_QUEUE_AGE_OLDEST) ; }
__inline BigREAL QUEUE_AGE_IN_RANGE(VECTOR_ARG *v,BigREAL low,BigREAL hi){ 
 return (*VGV->QUEUE_GETINFO)(v,low,hi,F_QUEUE_AGE_IN_RANGE) ; }
__inline BigREAL QUEUE_ATTRIB_MAX(VECTOR_ARG *v){ 
 return (*VGV->QUEUE_GETINFO)(v,0.0,0.0,F_QUEUE_ATTRIB_MAX) ; }
__inline BigREAL QUEUE_ATTRIB_MIN(VECTOR_ARG *v){ 
 return (*VGV->QUEUE_GETINFO)(v,0.0,0.0,F_QUEUE_ATTRIB_MIN) ; }
__inline BigREAL QUEUE_ATTRIB_AVERAGE(VECTOR_ARG *v,BigREAL x){ 
 return (*VGV->QUEUE_GETINFO)(v,x,0.0,F_QUEUE_ATTRIB_AVERAGE) ; }
__inline BigREAL QUEUE_ATTRIB_IN_RANGE(VECTOR_ARG *v,BigREAL lo,BigREAL hi){ 
 return (*VGV->QUEUE_GETINFO)(v,lo,hi,F_QUEUE_ATTRIB_IN_RANGE) ; }
__inline BigREAL QUEUE_ATTRIB_QUANTITY(VECTOR_ARG *v,BigREAL x){ 
 return (*VGV->QUEUE_GETINFO)(v,x,0.0,F_QUEUE_ATTRIB_QUANTITY) ; }
__inline BigREAL INIT_PURE_DELAY(int type,VAL_OFF deloff,VAL_OFF valoff,double deltime,double initval,double missval,TAB_TYPE *tab,TAB_TYPE *tab2) { 
 return (*VGV->INIT_PURE_DELAY)(type,deloff,valoff,deltime,initval,missval,tab,tab2) ; }
__inline BigREAL INIT_SINTEG(int type,VAL_OFF deloff,VAL_OFF valoff,double initval,double minval,double maxval,double quantval,double specval,double tolerance) { 
 return (*VGV->INIT_SINTEG)(type,deloff,valoff,initval,minval,maxval,quantval,specval,tolerance) ; }
__inline BigREAL STORE_PURE_DELAY(VAL_OFF deloff,double val,double outval,double deltime,BigREAL changerate,TAB_TYPE *tab) {  
 return (*VGV->STORE_PURE_DELAY)(deloff,val,outval,deltime,changerate,tab) ; }
__inline int COMPARE_SIMULT_VALUES(int storeflag)  {
 return (*VGV->COMPARE_SIMULT_VALUES)(storeflag) ; }
__inline BigREAL FIND_ZERO(VECTOR_ARG *vx,VECTOR_ARG *vsx,VECTOR_ARG *vz,VECTOR_ARG *vsz,VECTOR_ARG *xini,BigREAL fn,
 BigREAL fns,BigREAL tol,BigREAL fmax_iter,BigREAL method) { 
 return (*VGV->FIND_ZERO)(vx,vsx,vz,vsz,xini,fn,fns,tol,fmax_iter,method) ; }
__inline BigREAL INVERT_MATRIX(VECTOR_ARG *invmat,VECTOR_ARG *mat1,BigREAL fn) { 
 return (*VGV->INVERT_MATRIX)(invmat,mat1,fn) ; }
__inline BigREAL VECTOR_LOOKUP(VECTOR_ARG *in,BigREAL x,BigREAL xmin,BigREAL xmax,BigREAL mode) {  
 return (*VGV->VECTOR_LOOKUP)(in,x,xmin,xmax,mode) ; }
__inline void external_function(VV *val,int nval,int funcid) {
   (*VGV->EXTERNAL_FUNCTION)(val,nval,funcid) ; }
__inline int add_vecarg_info(ST_INDEX *stlist,int n) {
   return (*VGV->add_vecarg_info)(stlist,n) ; }
__inline  void vselect_error(int action,int errtype,int elmcount) {
  (*VGV->vselect_error)(action,errtype,elmcount) ; }


int VEFCC version_info(void)
 {
  return(COMPILED_VERSION) ;
  }

int VEFCC set_gv(GLOB_VARS *vgv) 
 {
  VGV = vgv ;
  if(!VGV || 
     VGV->vgv_magic_start != VGV_MAGIC_START ||
     VGV->vgv_magic_end != VGV_MAGIC_END)
     return 0 ;
  return 1 ;
  }


/*************************************************************************
  functions supported in vensim
  all need to be BigREAL for correct argument passing to work
 *************************************************************************/
BigREAL SHIFT_IF_TRUE(VECTOR_ARG *arr,BigREAL cond,BigREAL fnelm,BigREAL accum,
    BigREAL inival)
 {
  return VGV->SHIFT_IF_TRUE(arr,cond,fnelm,accum,inival) ;
  }

BigREAL ALLOC_P (BigREAL request,BigREAL priority,BigREAL width,BigREAL mp)
  {
   double half_width ;

   half_width = width/2 ;
  /* Consider degenerate cases of MP outside the priority distribution for
   the sector (which is a rectangle centered on PRIOR).  Otherwise just
   interpolate. */
  if ( mp >= priority + half_width)
      return(0.0) ;
  if ( mp <= priority - half_width )
     return(request) ;
  return(request * (priority + half_width - mp) / width) ;
  } /* alloc_p */

BigREAL ALLOCATE_BY_PRIORITY (VECTOR_ARG *allocation,VECTOR_ARG *request,VECTOR_ARG *priority,int size,
      BigREAL width,BigREAL supply)
 {
  int i ;
  BigREAL mp ;
  mp = (*VGV->MARKETP)(request,priority,size,width,supply) ;
  for(i=0;i<size;i++)
     allocation->vals[i] = (COMPREAL)ALLOC_P(request->vals[i],priority->vals[i],width,mp) ;
  return(allocation->vals[0]) ;
  }
BigREAL QUANTUM(BigREAL a,BigREAL b)
  {
   BigREAL q ;
   long j ;
   if(b < 1.0E-6) return(a) ;
   q = a/b ;
   if(q > -2E9 && q < 2E9) {
      j = (long)q ;
      return(b*j) ;
      }
   return(a) ;
   }
BigREAL MODULO(BigREAL a,BigREAL b)
  {
   long j ;
   if(b < 1.0E-6) return(a) ;
   j = (long)(a/b) ;
   return(a - b*j) ;
   }
BigREAL STEP(BigREAL height,BigREAL stime)
 {
  if(VGV->time_plus > stime)
     return(height) ;
  return(0.0) ;
  }
BigREAL RAMP(BigREAL slope,BigREAL starttime,BigREAL stoptime)
 {
  if(VGV->time_plus > starttime) {
     if(VGV->time_plus < stoptime)
        return(slope*(VGV->time-starttime)) ;
     if(stoptime > starttime)
        return(slope*(stoptime - starttime)) ;
     return(slope*(VGV->time-starttime)) ;
     }
  return(0.0) ;
  }
BigREAL PULSE(BigREAL ftime,BigREAL width) /* centered with time_plus */
  {
   if(width == 0)
      width = VGV->time_step ;
   if((VGV->time_plus > ftime) && (VGV->time_plus < (ftime + width)))
      return(1.0) ;
   return(0.0) ;
   }
BigREAL PULSE_TRAIN(BigREAL ftime,BigREAL width,BigREAL repeat,BigREAL stop) /* centered with time_plus */
  {
   long i ;
   if(VGV->time_minus > stop || VGV->time_plus < ftime)
      return 0 ;
   if(repeat < VGV->time_step)
      return 1.0 ;
   i = (long)((VGV->time_plus - ftime)/repeat) ;
   ftime += i*repeat ;
   if(width == 0)
      width = VGV->time_step ;
   if(VGV->time_plus < (ftime + width))
      return(1.0) ;
   return(0.0) ;
   }
BigREAL POWER(BigREAL base,BigREAL expon)
 {
  int iexpon ; 
  BigREAL texpon ;
  
  iexpon = (int)expon ;
  texpon = iexpon ;
  if(texpon == expon) { /* integer exponent */ 
     texpon = 1.0 ;    
     if(iexpon < 0) {
        iexpon = -iexpon ;
        base = 1.0/base ;
        }  
     for(;iexpon>0;iexpon--)
        texpon *= base ; 
     return(texpon) ;
     }
  if(base == 0.0 && expon > 0.0)
      return(0.0) ;
  return(exp(expon*log(base))) ;
  }
BigREAL GAMMA_LN(BigREAL xx)
 {    
  double x,tmp,ser;
 static double cof[6]={76.18009173,-86.50532033,24.01409822,
    -1.231739516,0.120858003e-2,-0.536382e-5};
 int j;

 x=xx-1.0;
 tmp=x+5.5;
 tmp -= (x+0.5)*log(tmp);
 ser=1.0;
 for (j=0;j<6;j++) {
   x += 1.0;
   ser += cof[j]/x;
   }
  return -tmp+log(2.50662827465*ser);
  }
BigREAL TIME_BASE (BigREAL intercept, BigREAL slope)
 {
  return(intercept + slope*VGV->time) ;
  }


BigREAL SAMPLE_IF_TRUE(double oldval,double cond,double newval)
 {
  if(cond == 0.0)
     return(oldval) ;
  return(newval) ;
  }
BigREAL VECTOR_ELM_MAP(VECTOR_ARG *in,BigREAL shift)
 {
  int ishift ; 
  COMPREAL *val ;
  if(shift < 0)
     ishift = (int)(shift - .5) ;  
  else
     ishift = (int)(shift + .5) ; 
  val = in->vals + ishift ;
  if(val < in->firstval ||
     val >= in->firstval + in->dim_info->tot_vol) {
     (*VGV->error_message)(VERROR,"VECTOR_ELM_MAP outside array bounds") ;
     return(NA) ;
     }
  return((BigREAL)(*val)) ;
  }
